/*
 * Copyright (c) 2011, Olivier MATZ <zer0@droids-corp.org>
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <aversive/pgmspace.h>

#include <stdio.h>
#include <inttypes.h>
#include <ctype.h>
#include <string.h>

#include <parse.h>
#include <xbee.h>

#include "parse_atcmd.h"

static int8_t
parse_atcmd(PGM_P tk, const char *buf, void *res)
{
	struct xbee_atcmd copy;
	struct token_atcmd_data ad;
	const struct xbee_atcmd *cmd;
	char bufcopy[32];
	uint8_t token_len = 0;

	memcpy_P(&ad, &((struct token_atcmd *)tk)->atcmd_data, sizeof(ad));

	while (!isendoftoken(buf[token_len]) &&
	       token_len < (sizeof(bufcopy)-1)) {
		bufcopy[token_len] = buf[token_len];
		token_len++;
	}
	bufcopy[token_len] = 0;

	/* XXX should be related to an xbee device */
	cmd = xbee_atcmd_lookup_desc(bufcopy);

	if (cmd == NULL)
		return -1;

	/* command must match flags */
	memcpy_P(&copy, cmd, sizeof(copy));
	if ((copy.flags & ad.atcmd_mask) != ad.atcmd_flags)
		return -1;
	/* store the address of object in structure */
	if (res)
		*(const struct xbee_atcmd **)res = cmd;

	return token_len;
}

static int8_t complete_get_nb_atcmd(PGM_P tk)
{
	struct token_atcmd_data ad;
	const struct xbee_atcmd *cmd;
	struct xbee_atcmd copy;
	int8_t cnt = 0;

	memcpy_P(&ad, &((struct token_atcmd *)tk)->atcmd_data, sizeof(ad));

	for (cmd = &xbee_atcmd_list[0], memcpy_P(&copy, cmd, sizeof(copy));
	     copy.name != NULL;
	     cmd++, memcpy_P(&copy, cmd, sizeof(copy))) {

		if ((copy.flags & ad.atcmd_mask) == ad.atcmd_flags)
			cnt++;
	}
	return cnt;
}

static int8_t complete_get_elt_atcmd(PGM_P tk, int8_t idx,
			      char *dstbuf, uint8_t size)
{
	struct token_atcmd_data ad;
	const struct xbee_atcmd *cmd;
	struct xbee_atcmd copy;
	int8_t cnt = 0;

	memcpy_P(&ad, &((struct token_atcmd *)tk)->atcmd_data, sizeof(ad));

	for (cmd = &xbee_atcmd_list[0], memcpy_P(&copy, cmd, sizeof(copy));
	     copy.name != NULL;
	     cmd++, memcpy_P(&copy, cmd, sizeof(copy))) {

		if ((copy.flags & ad.atcmd_mask) == ad.atcmd_flags) {
			if (cnt == idx) {
				memcpy_P(dstbuf, copy.desc, size);
				dstbuf[size-1] = '\0';

				return 0;
			}
			cnt++;
		}
	}
	return -1;
}

static int8_t
help_atcmd(PGM_P tk, char *dstbuf, uint8_t size)
{
	(void)tk;
	snprintf(dstbuf, size, "ATCMD");
	return 0;
}

struct token_ops token_atcmd_ops = {
	.parse = parse_atcmd,
	.complete_get_nb = complete_get_nb_atcmd,
	.complete_get_elt = complete_get_elt_atcmd,
	.get_help = help_atcmd,
};
